#!/bin/bash
# knoppix-networkmanager - A light-weight configuration tool
# for /etc/network/interfaces.
# (C) Klaus Knopper Jan. 2012

export TEXTDOMAINDIR="/usr/share/locale"
export TEXTDOMAIN=knoppix-networkmanager

# Configurable area

# Subkeys from /etc/network/interfaces (man 5 interfaces)
# [/etc/network/interfaces key] [stored in array] [display to the user]
CONFIG=( \
	wpa-ssid	WPA_SSID	"$(gettext "network name (essid)")" \
	wpa-driver	WPA_DRIVER	"$(gettext "WPA driver (normally 'wext')")" \
	wpa-key-mgmt	WPA_KEY_MGMT	"$(gettext "WPA encryption type")" \
	wpa-psk		WPA_PSK		"$(gettext "WPA password")" \
	wireless-key	WIRELESS_KEY 	"$(gettext "WEP encryption key")" \
	wireless-essid	WIRELESS_ESSID  "$(gettext "network name (essid)")" \
	wireless-mode	WIRELESS_MODE	"$(gettext "network mode (normally 'managed')")" \
	address		ADDRESS 	"$(gettext "static IP address")" \
	netmask		NETMASK 	"$(gettext "static network mask")" \
	gateway		GATEWAY 	"$(gettext "static gateway")" \
)

# Types of connection: [short name] [displayed name]
TYPES=( \
	lan	"$(gettext "wired network (LAN)")" \
	wlan	"$(gettext "wireless network (WLAN)")" \
	modem	"$(gettext "dialup (modem) connection")" \
)

SUBTYPES=( \
	wpa	"$(gettext "wireless WPA-encrypted connection")" \
	wep	"$(gettext "wireless WEP-encrypted connection")" \
	open	"$(gettext "wireless open/unencrypted connection")" \
)

CONNECTION_LABEL="$(gettext "%s connection %s")"
YES_LABEL="$(gettext "Yes")"
MANUAL_LABEL="$(gettext "manual")"
ESSID_LABEL="$(gettext "Network name (essid)")"
SCANNED_LABEL="$(gettext "Found")"
CONNECTION_DHCP_LABEL="$(gettext "Automatic address (DHCP)")"
MENU_TITLE="$(gettext "Network Connections")"
MENU_SELECT_CONNECTION="$(gettext "Please select connection")"
MENU_EDIT_CONNECTION="$(gettext "View/Modify connection")"
MENU_NEW_CONNECTION="$(gettext "Create new connection")"
MENU_DELETE_CONNECTION="$(gettext "Delete connection")"
MENU_DEFAULT_CONNECTION="$(gettext "Set as standard/auto connection")"
MENU_CONNECT_CONNECTION="$(gettext "Connect now")"
MENU_DISCONNECT_CONNECTION="$(gettext "Disconnect now")"
MENU_CONNECTION_INFO="$(gettext "connection info")"
CONNECTION_NAME_LABEL="$(gettext "Name of connection")"
CONNECTION_TYPE_LABEL="$(gettext "Type of connection")"
NEW_CONNECTION_TEMPLATE="$(gettext "newconnection")"
CONNECTION_TYPE=() # Contains the type of connection as in TYPES[]
CONNECTION_SUBTYPE=() # Contains the subtype of connection as in SUBTYPES[]
CONNECTION_NAME=() # Contains the iface 'name' name of the connection
CONNECTION_DHCP=() # True if connection is setup via dhcp
CONNECTION_SAVE_LABEL="$(gettext "Save connection")"
CONNECTION_NOSAVE_LABEL="$(gettext "Don't save")"
ERROR_TITLE="$(gettext "Error")"
ERROR_CONNECTION_NAME="$(gettext "Name may only contain a-z and 0-9, and no spaces")"
ERROR_CONNECTION_DUPLICATE="$(gettext "Connection name already exists")"
CONNECTION_REALLY_DELETE_LABEL="$(gettext "Really delete connection")"
CONNECTION_NODELETE_LABEL="$(gettext "Don't delete")"
CONNECTED_LABEL="$(gettext "connected")"
MENU_QUIT="$(gettext "Quit")"
MENU_RETURN="$(gettext "Return")"
PHYS_NETWORK_DEVICE="$(gettext "physical network device")"
ENTER_CONNECTION_NAME="$(gettext "enter connection name")"

# End of configurable area

PATH="/bin:/sbin:/usr/bin:/usr/sbin"
export PATH

[ "`id -u`" != "0" ] && exec sudo -H "$0" "$@"

TMP="/tmp/knoppix-networkmanager.tmp$$"

NETWORK_MANAGEMENT=""
EXCLUDE_DEVICES=""
MATCH_DEVICES=""

AUTO=""
CHANGED=""

CONFIG_ARRAYS=()
for ((i=0; i<${#CONFIG[@]}/3; i++)); do
 CONFIG_ARRAYS[$i]="${CONFIG[$((i*3+1))]}"
done

# One-time dialog settings
if [ -z "$DISPLAY" ]; then
 # Quick check for support of "--no-ok" in dialog
 NOOK=""
 case "$(dialog --help 2>&1)" in *--no-ok*) export NOOK="--no-ok";; esac
 export DIALOGOPTS="$NOOK --visit-items --no-cancel --begin 0 0 --no-shadow"
 [ -f /etc/adriane/dialogrc ] && export DIALOGRC="/etc/adriane/dialogrc"
fi

# dialog_input title info [old_text]
dialog_input(){
 local title="$1" info="$2" old="$3"
 if [ -n "$DISPLAY" ]; then
  zenity --entry --width=550 --title="$title" --text="$info" --entry-text="$old" 1>&2 2>/dev/null
 else
  dialog --title "$title" --inputbox "$info" 10 75 "$old"
 fi
}

# dialog_menu title wrapped_text menuitem1 description1 ...
dialog_menu(){
 local title="$1" info="$2"
 shift 2
 if [ -n "$DISPLAY" ]; then
  for item in "$@"; do echo "$item"; done | { zenity --list --width=550 --height=280 --hide-header --title="$title" --text="$info" --column="" --column="" 1>&2 2>/dev/null; }
 else
  dialog --title "$title" --menu "$info" 18 75 12 "$@"
 fi
}

# error_message "text"
error_message(){
 if [ -n "$DISPLAY" ]; then
  zenity --width=550 --height=280 --error --title="$ERROR_TITLE" --text="$1" 1>&2 2>/dev/null
 else
  dialog --menu "$ERROR_TITLE" 10 75 5 "" "$1"
 fi
}

get_type(){
 local count
 for ((count=0; count<${#TYPES[@]}/2; count++)); do
  case "${TYPES[$((count*2))]}" in $1) echo "${TYPES[$((count*2+1))]}" ; return 0 ;; esac
 done
 return 1
}

get_subtype(){
 local count
 for ((count=0; count<${#SUBTYPES[@]}/2; count++)); do
  case "${SUBTYPES[$((count*2))]}" in $1) echo "${SUBTYPES[$((count*2+1))]}" ; return 0 ;; esac
 done
 return 1
}

# get by key
get_config_array(){
 local count
 for ((count=0; count<${#CONFIG[@]}/3; count++)); do
  case "${CONFIG[$((count*3))]}" in $1) echo "${CONFIG[$((count*3+1))]}" ; return 0 ;; esac
 done
 return 1
}

# get by array name
get_config_key(){
 local count
 for ((count=0; count<${#CONFIG[@]}/3; count++)); do
  case "${CONFIG[$((count*3+1))]}" in $1) echo "${CONFIG[$((count*3))]}" ; return 0 ;; esac
 done
 return 1
}

# get by array name
get_config_description(){
 local count
 for ((count=0; count<${#CONFIG[@]}/3; count++)); do
  case "${CONFIG[$((count*3+1))]}" in $1) echo "${CONFIG[$((count*3+2))]}" ; return 0 ;; esac
 done
 return 1
}

# Zero Global Arrays
reset_config(){
 local array i=1
 while ((i < ${#CONFIG[@]})); do
  array="${CONFIG[$i]}"
  eval "${array}=()"
  let i=i+3
 done
 CONNECTION_TYPE=()
 CONNECTION_NAME=()
 CONNECTION_DHCP=()
 connection_count=0
}

# Convert /etc/network/interfaces into our own array-based format
read_interfaces(){
 local iface device inet method line relax key val count config
 reset_config
 while read iface device inet method relax; do
  case "$iface" in iface)
   case "$device" in lo) continue;; esac
   if [ -n "$EXCLUDE_DEVICES" ]; then
    case "$device" in $EXCLUDE_DEVICES) continue;; esac
   fi
   if [ -n "$MATCH_DEVICES" ]; then
    case "$device" in $MATCH_DEVICES) true;; *)continue;; esac
   fi
   CONNECTION_NAME[$connection_count]="$device"
   CONNECTION_TYPE[$connection_count]="lan"
   case "$method" in
    dhcp)   CONNECTION_DHCP[$connection_count]="y" ;;
    wvdial) CONNECTION_TYPE[$connection_count]="modem" ;;
   esac
   while read key val; do
    [ -n "$key" ] || break
    case "$key" in \#*) continue ;; esac
    case "$key" in
     wpa*)
      CONNECTION_TYPE[$connection_count]="wlan"
      [ -n "${CONNECTION_SUBTYPE[$connection_count]}" ] || CONNECTION_SUBTYPE[$connection_count]="wpa" ;;
     wireless*)
      CONNECTION_TYPE[$connection_count]="wlan"
      [ -n "${CONNECTION_SUBTYPE[$connection_count]}" ] || CONNECTION_SUBTYPE[$connection_count]="open"
      case "$key" in wireless-key) CONNECTION_SUBTYPE[$connection_count]="wep" ;; esac ;;
     esac
    count=0
    while ((count < ${#CONFIG[@]})); do
     if [ -n "${CONFIG[$count]}" ]; then
      case "$key" in ${CONFIG[$count]})
       IFS="" eval ${CONFIG[$((count+1))]}[$connection_count]="\$val" ; break ;;
      esac
     fi
     let count+=3
    done
   done
   let connection_count++
   ;;
  auto)
   AUTO="$AUTO $device"
  ;;
  esac
 done </etc/network/interfaces
}

bailout(){
 rm -f "$TMP"
 exit $1
}

# Simple shell grep
stringinfile(){
 case "$(cat $2)" in *$1*) return 0;; esac
 return 1
}

# Read into variable: select_connection variable
select_connection(){
 local MENU=( i "$MENU_CONNECTION_INFO" )
 local ccount mcount=2 connected rc
 read_interfaces
 rm -f "$TMP".nmtool
 for ((ccount=0; ccount < connection_count; ccount++)); do
  case "${CONNECTION_NAME[$ccount]}" in lo) continue ;; esac
  MENU[$((mcount++))]="$ccount"
  check_connected "$ccount" && connected=" ($CONNECTED_LABEL)" || connected=""
  MENU[$((mcount++))]="$(printf "$CONNECTION_LABEL" "${CONNECTION_TYPE[$ccount]}" "${CONNECTION_NAME[$ccount]}$connected")"
 done
 rm -f "$TMP".nmtool
 MENU[$((mcount++))]="n"
 MENU[$((mcount++))]="$MENU_NEW_CONNECTION"
 MENU[$((mcount++))]="q"
 MENU[$((mcount++))]="$MENU_QUIT"
 dialog_menu "$MENU_TITLE" "$MENU_SELECT_CONNECTION" "${MENU[@]}" 2>"$TMP" 
 rc="$?"
 read "$1" <"$TMP" ; rm -f "$TMP"
 return "$rc"
}

# sort_menu defaultitem item1 description1 ...
# sorts menu and saves it in the global MENU_SORTED array
MENU_SORTED=()
sort_menu(){
 unset IFS
 local defaultitem="$1"; shift
 local i=0 count=0
 local MENU_UNSORTED=("$@")
 MENU_SORTED=()
 # First round: find default item+description
 for ((i=0; i<${#MENU_UNSORTED[@]}; i+=2)); do
  if [ x"${MENU_UNSORTED[$i]}" = x"$defaultitem" ]; then
   MENU_SORTED[$((count++))]="${MENU_UNSORTED[$i]}"
   MENU_SORTED[$((count++))]="${MENU_UNSORTED[$((i+1))]}"
   break
  fi
 done
 # Now, add remaining items
 for ((i=0; i<${#MENU_UNSORTED[@]}; i+=2)); do
  if [ x"${MENU_UNSORTED[$i]}" != x"$defaultitem" ]; then
   MENU_SORTED[$((count++))]="${MENU_UNSORTED[$i]}"
   MENU_SORTED[$((count++))]="${MENU_UNSORTED[$((i+1))]}"
  fi
 done
}

# edit_connection number [default item]
edit_connection(){
 local changed="" rc=0
 local DEFAULTACTION="$2"
 # Insert defaults
 [ -n "${WPA_DRIVER[$1]}" ] || WPA_DRIVER[$1]="wext"
 [ -n "${WPA_WPA_KEY_MGMT[$1]}" ] || WPA_KEY_MGMT[$1]="WPA-PSK"
 [ -n "${WIRELESS_MODE[$1]}" ] || WIRELESS_MODE[$1]="managed"
 [ -n "${NETMASK[$1]}" ] || NETMASK[$1]="255.255.255.0"
 while true; do
  local MENU=()
  local item val mcount=0
  MENU[$((mcount++))]="c"; MENU[$((mcount++))]="$MENU_CONNECT_CONNECTION"
  MENU[$((mcount++))]="n"; MENU[$((mcount++))]="$CONNECTION_NAME_LABEL: ${CONNECTION_NAME[$1]}"
  MENU[$((mcount++))]="t"
  if [ -n "${CONNECTION_SUBTYPE[$1]}" ]; then
   MENU[$((mcount++))]="$CONNECTION_TYPE_LABEL: ${CONNECTION_TYPE[$1]}/${CONNECTION_SUBTYPE[$1]}"
  else
   MENU[$((mcount++))]="$CONNECTION_TYPE_LABEL: ${CONNECTION_TYPE[$1]}"
  fi
  case "${CONNECTION_TYPE[$1]}" in lan|wlan) 
   MENU[$((mcount++))]="h"
   if [ -n "${CONNECTION_DHCP[$1]}" ]; then
    MENU[$((mcount++))]="$CONNECTION_DHCP_LABEL: $YES_LABEL"
   else
    MENU[$((mcount++))]="$CONNECTION_DHCP_LABEL: $MANUAL_LABEL"
   fi ;;
  esac
  for item in "${CONFIG_ARRAYS[@]}"; do
   # Skip items that don't apply to type of connection
   case "$item" in WPA_*|WIRELESS_*) case "${CONNECTION_TYPE[$1]}" in wlan) ;; *) continue ;; esac ;; esac
   case "$item" in WPA_*) case "${CONNECTION_SUBTYPE[$1]}" in wpa) ;; *) continue ;; esac ;; esac
   case "$item" in WIRELESS_*) case "${CONNECTION_SUBTYPE[$1]}" in wep|open) ;; *) continue ;; esac ;; esac
   case "$item" in ADDRESS|NETMASK|GATEWAY) [ -n "${CONNECTION_DHCP[$1]}" ] && continue ;; esac
   IFS="" eval val="\${${item}[$1]}"
   MENU[$((mcount++))]="$(get_config_key $item)"
   MENU[$((mcount++))]="$(get_config_description $item): $val"
  done
#  MENU[$((mcount++))]="a"; MENU[$((mcount++))]="$MENU_DEFAULT_CONNECTION"
#  MENU[$((mcount++))]="d"; MENU[$((mcount++))]="$MENU_DISCONNECT_CONNECTION"
  MENU[$((mcount++))]="x"; MENU[$((mcount++))]="$MENU_DELETE_CONNECTION"
  rm -f "$TMP"
  MENU[$((mcount++))]="q"; MENU[$((mcount++))]="$MENU_RETURN"
  if [ -n "$DEFAULTACTION" ]; then
   sort_menu "$DEFAULTACTION" "${MENU[@]}"
   MENU=( "${MENU_SORTED[@]}" )
  fi
  dialog_menu "$MENU_TITLE" "$MENU_EDIT_CONNECTION" "${MENU[@]}" 2>"$TMP"
  rc="$?"
  read selection <"$TMP" ; rm -f "$TMP"
  [ "$rc" = 0 ] || break
  DEFAULTACTION="$selection"
  case "$selection" in
   n) edit_name $1 ;; # Rename does not need save_connection
   x) delete_connection $1 && return 0;; # Deleting is immediate
   c) if [ -z "$changed" -a -z "$CHANGED" ] || save_connection "$1"; then
       changed=""; CHANGED=""
       connect_connection $1
       break
      fi ;;
   t) edit_type $1 && changed="true" ;;
   h) edit_dhcp $1 && changed="true" ;;
   *ssid) edit_essid $1 && changed="true" ;;
   q) break ;;
   *) edit_option $1 "$selection" && changed="true" ;;
  esac
 done
 [ -n "$changed" -o -n "$CHANGED" ] && save_connection "$1"
 return "$rc"
}

# connect_connection number
connect_connection(){
 local name="${CONNECTION_NAME[$1]}" i rc
 [ -n "$name" ] || return 1
 # Remove all "auto" lines except for loopback, then
 # add an auto line for current connection
 sed -i -e '/^auto /{/auto.* lo/!d}' -e '/^iface *'"$name"' */i auto\ '"$name" \
     /etc/network/interfaces
 case "$NETWORK_MANAGEMENT" in
  networkmanager)
   if pidof -x nm-system-settings >/dev/null 2>&1; then
    killall -HUP nm-system-settings
   elif pidof -x NetworkManager >/dev/null 2>&1 && [ -x /etc/init.d/network-manager ]; then
    /etc/init.d/network-manager force-reload
   fi ;;
  ifupdown)
   ifdown "$name" >/dev/null 2>&1 ; ifup "$name" ;;
 esac
 sleep 1
}

# check_connected number
check_connected(){
 local name="${CONNECTION_NAME[$1]}"
 local stateinfo=""
 case "$NETWORK_MANAGEMENT" in
  networkmanager)
   [ -r "$TMP".nmtool ] || nm-tool >"$TMP".nmtool 2>/dev/null
   stateinfo="$(sed -n '/Device:.*\('"$name"'\)/,/State: .*/{/State:[\t ]*connected/p}' "$TMP".nmtool)" ;;
  ifupdown)
   stateinfo="$(route -n | awk '$NF ~ "'"${name}"'"{print}')" ;;
 esac
 [ -n "$stateinfo" ] && return 0
 return 1
}

# edit_option number option
edit_option(){
 local rc array="$(get_config_array "$2")" result description old
 [ -n "$array" ] || return 1
 eval old="\"\${$array[$1]}\""
 description="$(get_config_description "$array")"
 dialog_input "$MENU_TITLE" "$description" "$old" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"; return "$rc"
 fi
 read result <"$TMP"; rm -f "$TMP"
 eval "$array[$1]"="\"$result\""
 return "$rc"
}

# edit_essid number
edit_essid(){
 local ssid rc=1
 case "${CONNECTION_SUBTYPE[$1]}" in
  wpa) 
   select_essid ssid "${WPA_SSID[$1]}"; rc="$?"
   [ "$rc" = 0 ] && WPA_SSID[$1]="$ssid"
   ;;
  wep|open)
   select_essid ssid "${WIRELESS_ESSID[$1]}"; rc="$?"
   [ "$rc" = 0 ] && WIRELESS_ESSID[$1]="$ssid"
   ;;
 esac
 return "$rc"
}

# select_essid essid_variable [old]
select_essid(){
 local MENU=()
 local selection essid="" encryption="" line mcount=0 rc
 rm -f "$TMP"
 while read line; do
  case "$line" in
  *ESSID:*)
   if [ -n "$essid" ]; then # Add previous entry
    MENU[$((mcount++))]="$essid"
    MENU[$((mcount++))]="$SCANNED_LABEL: '$essid'$encryption"
    encryption=""
   fi
   essid="${line#*\"}"; essid="${essid%\"*}";;
  *Encryption\ key:on*) encryption=" (WEP)";;
  *Encryption\ key:*) encryption="";;
  *Authentication\ *:*\ PSK*) encryption=" (WPA/PSK)";;
  esac
 done <<EOT
$(LC_MESSAGES=C LANG=C iwlist scan 2>/dev/null)
EOT
 if [ -n "$essid" ]; then # Add last entry
  MENU[$((mcount++))]="$essid"
  MENU[$((mcount++))]="$SCANNED_LABEL: '$essid'$encryption"
 fi
 MENU[$((mcount++))]="m"
 MENU[$((mcount++))]="$MANUAL_LABEL"
 if [ -n "$2" ]; then
  sort_menu "$2" "${MENU[@]}"
  MENU=( "${MENU_SORTED[@]}" )
 fi
 dialog_menu "$MENU_TITLE" "$ESSID_LABEL" "${MENU[@]}" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"; return "$rc"
 fi
 read selection <"$TMP"
 if [ -n "$selection" -a "$selection" != "m" ]; then
  read "$1" <"$TMP" ; rm -f "$TMP"; return 0
 else
  dialog_input "$MENU_TITLE" "$ESSID_LABEL" "$2" 2>"$TMP"; rc="$?"
  if [ "$rc" != 0 ]; then
   rm -f "$TMP"; return "$rc"
  fi
  read "$1" <"$TMP" ; rm -f "$TMP"; return 0
  return 0
 fi
}

# select_type type_variable subtype_variable [old_type] [old_subtype]
select_type(){
 local MENU=( "${TYPES[@]}" )
 local newtype newsubtype rc
 rm -f "$TMP"
 local DEFAULTITEM=()
 if [ -n "$3" ]; then
  sort_menu "$3" "${MENU[@]}"
  MENU=( "${MENU_SORTED[@]}" )
 fi
 dialog_menu "$MENU_TITLE" "$NEW_TYPE" "${MENU[@]}" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"; return "$rc"
 fi
 read "$1" <"$TMP" ; rm -f "$TMP"
 if [ "${!1}" = "wlan" ]; then
  MENU=( "${SUBTYPES[@]}" )
  if [ -n "$4" ]; then
   sort_menu "$4" "${MENU[@]}"
   MENU=( "${MENU_SORTED[@]}" )
  fi
  dialog_menu "$MENU_TITLE" "$NEW_SUBTYPE" "${MENU[@]}" 2>"$TMP"; rc="$?"
  if [ "$rc" != 0 ]; then
   rm -f "$TMP"; return "$rc"
  fi
  read "$2" <"$TMP" ; rm -f "$TMP"
 else
  eval "$2"=""
 fi
 return 0
}

# edit_name $1
edit_name(){
 local NAME ccount rc
 while true; do
  NAME="${CONNECTION_NAME[$1]}"
  if [ -z "$NAME" ]; then
   local PHYSDEV=() physcount=0 d vid did desc name_selection
   for d in $(echo /sys/class/net/{eth*,w*}); do
    [ -d "$d" ] || continue
    if [ -n "$EXCLUDE_DEVICES" ]; then
     case "${d##*/}" in $EXCLUDE_DEVICES) continue;; esac
    fi
    if [ -n "$MATCH_DEVICES" ]; then
     case "${d##*/}" in $MATCH_DEVICES) true;; *) continue;; esac
    fi
    read vid < "$d/device/vendor" 2>/dev/null
    read did < "$d/device/device" 2>/dev/null
    if [ -n "$vid" -a -n "$did" ]; then
     desc="$(LC_MESSAGES=C lspci -vmm -d "${vid}:${did}" | awk -F: '/^Vendor:|^Device:/{printf "%s",$NF}' | sed 's|\t| |g')"
    else
     desc="$PHYS_NETWORK_DEVICE"
    fi
    PHYSDEV[$((physcount++))]="${d##*/}"
    PHYSDEV[$((physcount++))]="${desc}"
   done
   PHYSDEV[$((physcount++))]="n"
   PHYSDEV[$((physcount++))]="$ENTER_CONNECTION_NAME"
   dialog_menu "$MENU_TITLE" "$NEW_CONNECTION" "${PHYSDEV[@]}" 2>"$TMP"; rc="$?"
   if [ "$rc" != 0 ]; then
    rm -f "$TMP"; return "$rc"
   fi
   read name_selection <"$TMP"
   case "$name_selection" in
    n)
     NAME="${NEW_CONNECTION_TEMPLATE}$1"
     dialog_input "$MENU_TITLE" "$NEW_CONNECTION" "$NAME" 2>"$TMP"; rc="$?"
     if [ "$rc" != 0 ]; then
      rm -f "$TMP"; return "$rc"
     fi
     read NAME <"$TMP"; rm -f "$TMP"
     ;;
    *) NAME="$name_selection" ;;
   esac
  fi
  if echo "$NAME" | grep -qi -e '^[a-z0-9]*$'; then
   [ "${CONNECTION_NAME[$1]}" = "$NAME" ] && return 0
   for ((ccount=0; ccount < connection_count; ccount++)); do
    if [ "$NAME" = "${CONNECTION_NAME[$ccount]}" ]; then
     error_message "$NAME: $ERROR_CONNECTION_DUPLICATE"
     continue 2
    fi
   done
   # Here should be another question "Really rename?"
   [ -n "${CONNECTION_NAME[$1]}" ] && rename_connection "${CONNECTION_NAME[$1]}" "$NAME"
   CONNECTION_NAME[$1]="$NAME"
   break
  else
   error_message "$ERROR_CONNECTION_NAME"
  fi
 done
 return "$rc"
}

# rename_connection old new
rename_connection(){
 sed -i -e 's|^iface *'"$1"' *|iface '"$2"' |g' \
        -e 's|^auto *'"$1"' *$|auto '"$2"'|g' \
        -e 's|^allow-hotplug *'"$1"' *$|allow-hotplug '"$2"'|g' \
     /etc/network/interfaces
}

# edit_type number  
edit_type(){
 local TYPE SUBTYPE rc
 select_type TYPE SUBTYPE "${CONNECTION_TYPE[$1]}"; rc="$?"
 if [ "$rc" = 0 ]; then
  CONNECTION_TYPE[$1]="$TYPE"
  CONNECTION_SUBTYPE[$1]="$SUBTYPE"
 fi
 return "$rc"
}

# select_dhcp result_variable [old]
select_dhcp(){
 local MENU=( y "$YES_LABEL" m "$MANUAL_LABEL" )
 local rc
 if [ -n "$2" ]; then
  sort_menu "$2" "${MENU[@]}"
  MENU=( "${MENU_SORTED[@]}" )
 fi
 rm -f "$TMP"
 dialog_menu "$CONNECTION_DHCP_LABEL" "${DEFAULTITEM[@]}" --menu "$CONNECTION_DHCP_LABEL" 10 75 5 "${MENU[@]}" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"; return "$rc"
 fi
 read "$1" <"$TMP" ; rm -f "$TMP"
 return 0
}

# edit_dhcp $1
edit_dhcp(){
 local DHCP rc
 select_dhcp DHCP "${CONNECTION_DHCP[$1]}"; rc="$?"
 if [ "$rc" = 0 ]; then
  [ "$DHCP" = "y" ] && CONNECTION_DHCP[$1]="y" || CONNECTION_DHCP[$1]=""
 fi
 return "$rc"
}

# save_connection $1
save_connection(){
 local MENU=( y "$CONNECTION_SAVE_LABEL '${CONNECTION_NAME[$1]}'" n "$CONNECTION_NOSAVE_LABEL" )
 local item val result rc
 dialog_menu "$CONNECTION_SAVE_LABEL" " " "${MENU[@]}" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"
  return 1
 fi
 read result <"$TMP"; rm -f "$TMP"
 [ "$result" = "y" ] || return 1
 echo -n "iface ${CONNECTION_NAME[$1]} inet " >>"$TMP"
 if [ -n "${CONNECTION_DHCP[$1]}" ]; then
  echo "dhcp"
 elif [ "${CONNECTION_TYPE[$1]}" = "lan" -o "${CONNECTION_TYPE[$1]}" = "wlan" ]; then
  echo "static"
 else
  echo "manual"
 fi >>"$TMP"
 for item in "${CONFIG_ARRAYS[@]}"; do
  # Skip items that don't apply to type of connection
  case "$item" in WPA_*|WIRELESS_*) case "${CONNECTION_TYPE[$1]}" in wlan) ;; *) continue ;; esac ;; esac
  case "$item" in WPA_*) case "${CONNECTION_SUBTYPE[$1]}" in wpa) ;; *) continue ;; esac ;; esac
  case "$item" in WIRELESS_*) case "${CONNECTION_SUBTYPE[$1]}" in wep|open) ;; *) continue ;; esac ;; esac
  case "$item" in ADDRESS|NETMASK|GATEWAY) [ -n "${CONNECTION_DHCP[$1]}" ] && continue ;; esac
  IFS="" eval val="\${${item}[$1]}"
  [ -n "$val" ] && echo -e "\t$(get_config_key $item) $val"
 done >>"$TMP"
 # Workaround: For hidden ESSIDs and WPA, scan for accesspoint & essid
 if [ "${CONNECTION_TYPE[$1]}" = "wlan" -a "${CONNECTION_SUBTYPE[$1]}" = "wpa" ]; then
  echo -e "\twpa-ap-scan 1\n\twpa-scan-ssid 1" >>"$TMP"
 fi
 echo "" >>"$TMP"
 if grep -q '^iface '"${CONNECTION_NAME[$1]}"' ' /etc/network/interfaces; then
  # Replace old entry
   sed -i '/^iface '"${CONNECTION_NAME[$1]}"' /,/^$/{/^$/r '"$TMP"'
d}' /etc/network/interfaces; rc="$?"
 else
  # Add entry
  echo "" >>/etc/network/interfaces
  cat "$TMP" >>/etc/network/interfaces; rc="$?"
 fi
 rm -f "$TMP"
 return "$rc"
}

# delete_connection number
delete_connection(){
 local MENU=( y "$CONNECTION_REALLY_DELETE_LABEL '${CONNECTION_NAME[$1]}'" n "$CONNECTION_NODELETE_LABEL" )
 local item val result rc
 dialog_menu "$CONNECTION_REALLY_DELETE_LABEL" " " "${MENU[@]}" 2>"$TMP"; rc="$?"
 if [ "$rc" != 0 ]; then
  rm -f "$TMP"
  return 1
 fi
 read result <"$TMP"; rm -f "$TMP"
 [ "$result" = "y" ] || return 1
 sed -i -e '/^iface '"${CONNECTION_NAME[$1]}"' /,/^$/d' \
        -e '/^auto *'"$1"' *$/d' \
        -e '/^allow-hotplug *'"$1"' *$/d' \
     /etc/network/interfaces
 return $?
}

new_connection(){
 local current rc
 edit_name "$connection_count"; rc="$?"
 [ "$rc" != 0 ] && return "$rc"
 edit_type "$connection_count"; rc="$?"
 [ "$rc" != 0 ] && return "$rc"
 current="$connection_count"
 # Insert defaults
 CONNECTION_DHCP[$current]="yes"
 let connection_count++
 CHANGED="yes"
 edit_connection "$current" h; rc="$?"
 CHANGED=""
 return "$rc"
}

# connection_info (no parms)
connection_info(){
 local MENU=() mcount=0
 case "$NETWORK_MANAGEMENT" in
  networkmanager) nm-tool 2>/dev/null | sed -e '/^$/d' ;;
  ifupdown)       ifconfig ; route -n ;;
 esac | { while read line; do
    MENU[$((mcount++))]=" "
    MENU[$((mcount++))]="$line"
    echo xxx "$line"
   done
   dialog_menu "$MENU_TITLE" " " "${MENU[@]}"; }
}

### MAIN

# Check for kind of connection we are using
if pidof -x NetworkManager >/dev/null 2>&1 || pidof -x nm-system-settings >/dev/null 2>&1; then
 NETWORK_MANAGEMENT="networkmanager"
elif type -a ifup >/dev/null 2>&1; then
 NETWORK_MANAGEMENT="ifupdown"
else
 NETWORK_MANAGEMENT=""
fi
 
# Parse commandline arguments
action="add"
for arg in "$@"; do
 case "$arg" in
  -x) action="exclude"; continue ;;
  *)  case "$action" in
       exclude) [ -n "$EXCLUDE_DEVICES" ] && EXCLUDE_DEVICES="${EXCLUDE_DEVICES}|${arg}" || EXCLUDE_DEVICES="${arg}" ;;
       add)     [ -n "$MATCH_DEVICES" ]   && MATCH_DEVICES="${MATCH_DEVICES}|${arg}" || MATCH_DEVICES="${arg}" ;;
      esac
      action="add";;
 esac
done

# Main Loop
while select_connection SELECTION; do
 case "$SELECTION" in
  n) new_connection ;;
  i) connection_info ;;
  [0-9]*) edit_connection "$SELECTION" ;;
  *) break ;;
 esac
done

exit 0
